/*
 * File         : ActiveUser.cpp
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 24 18:06:26 1998
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1998/03/17 22:01:14 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include "ActiveUser.h"
#include <ace/Auto_Ptr.h>

ActiveUser::
ActiveUser( ClientHandler* ch )
: chandler_( ch ), destroy_( false ), mb_( 0 ), msg_( 0 ), user_info_( 0 ), ip_address_( 0 )
{
	ACE_INET_Addr addr;
	if( chandler_->peer().get_remote_addr( addr ) == -1 )
		ACE_ERROR(( LM_ERROR, "(%t) [!](ActiveUser::ActiveUser) address error\n" ));

	ip_address_ = htonl( addr.get_ip_address() );
	in_addr user_addr;
	user_addr.s_addr = ntohl( ip_address_ );
	ACE_DEBUG(( LM_DEBUG, "(%t) key: %u, IP address: %s\n",	ip_address_, inet_ntoa( user_addr ) ));
}

int ActiveUser::
service( ACE_Time_Value* tv )
{
	if( au_lock_.acquire() == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ActiveUser::service) acquire error\n"), -1 );

	if( mqueue_.dequeue_head( mb_, tv ) == -1 )
	{
		au_lock_.release();
		ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ActiveUser::service) dequeue_head\n"), -1 );
	}
	msg_ = ( DSMessage* )mb_->rd_ptr();
	mb_->release();
	msg_->execute();
	delete msg_;
	if( destroy_ )
		delete this;
	else
		au_lock_.release();

	return 0;
}

int ActiveUser::
send_msg( DirectoryProtocol::ProtoCode code, void* params, size_t param_size )
{
	DSMessage* msg;
	if( SDSProtocol::instance()->get( code, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ActiveUser::send_msg) %d protocol error\n", code ), -1 );

	ACE_DEBUG(( LM_DEBUG, "(%t) (ActiveUser::send) User: 0x%x, msg: 0x%x\n", this, msg ));
	auto_ptr<DSMessage> auto_msg( msg );
	if( msg->set( params, param_size ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ActiveUser::send_msg) %d setting error\n", code ), -1 );

	if( msg->send( chandler_->peer() ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ActiveUser::send_msg) msg 0x%x send error, handle %d\n",
						  msg, chandler_->peer().get_handle() ), -1 );

	return 0;
}

int ActiveUser::
bcast_msg( DirectoryProtocol::ProtoCode code, void* params, size_t param_size )
{
	DSMessage* msg;
	if( SDSProtocol::instance()->get( code, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ActiveUser::bcast_msg) %d protocol error\n", code ), -1 );

	ACE_DEBUG(( LM_DEBUG, "(%t) (ActiveUser::bcast_msg) User: 0x%x, msg: 0x%x\n", this, msg ));
	auto_ptr<DSMessage> auto_msg( msg );
	if( msg->set( params, param_size ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ActiveUser::bcast_msg) %d setting error\n", code ), -1 );

	DSActiveUserIterator it( *SDSActiveUserRep::instance() );
	DSActiveUserIterator end( *SDSActiveUserRep::instance(), 1 );
	while( it != end )
	{
		it.acquire_read();
		if( msg->send( ( *it ).int_id_->chandler_->peer() ) == -1 )
			ACE_ERROR(( LM_ERROR, "(%t) [!](ActiveUser::bcast_msg) send error, handle %d\n",
						( *it ).int_id_->chandler_->peer().get_handle() ));
		it++;
		it.release();
	}
	return 0;
}

int ActiveUser::
bcast( DSMessage* msg )
{
	ACE_DEBUG(( LM_DEBUG, "(%t) (ActiveUser::bcast) User: 0x%x, msg: 0x%x\n", this, msg ));
	DSActiveUserIterator it( *SDSActiveUserRep::instance() );
	DSActiveUserIterator end( *SDSActiveUserRep::instance(), 1 );
	while( it != end )
	{
		it.acquire_read();
		if( msg->send( ( *it ).int_id_->chandler_->peer() ) == -1 )
			ACE_ERROR(( LM_ERROR, "(%t) [!](ActiveUser::broadcast) msg send, handle %d\n",
						( *it ).int_id_->chandler_->peer().get_handle() ));
		it++;
		it.release();
	}
	return 0;
}
