/*
 * File         : ActiveUser.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/02/28 00:57:28 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __ACTIVEUSER_H )
#define __ACTIVEUSER_H

#include "DSSingletons.h"
#include "ClientHandler.h"
#include "ActiveUserRep.h"
#include "UserInfo.h"
#include "Participant.h"

#include <ace/Synch.h>
#include <ace/Message_Queue.h>

class ActiveUser
: public Participant
{
public:
	typedef ACE_Recursive_Thread_Mutex ActiveUserMutex;

	ActiveUser( ClientHandler* ch );
	~ActiveUser();
	int service( ACE_Time_Value* tv = 0 );
	int putq( ACE_Message_Block* mb, ACE_Time_Value* tv = 0 );
	void schedule_destruction() { destroy_ = true; }
	void info( UserInfo* ui ) { user_info_ = ui; }
	const UserInfo* info() const { return user_info_; }
	u_long get_ip_address();

	int send( DSMessage* msg );
	int bcast( DSMessage* msg );
	int send_msg( DirectoryProtocol::ProtoCode code,
				  void* params,
				  size_t param_size = 0 );
	int bcast_msg( DirectoryProtocol::ProtoCode code,
				   void* params,
				   size_t param_size = 0 );

	ParticipantKey key();

protected:
	ACE_Message_Queue<ACE_MT_SYNCH> mqueue_;
	ActiveUserMutex au_lock_;
	ACE_Message_Block* mb_;
	DSMessage* msg_;
	ClientHandler* chandler_;
	UserInfo* user_info_;
	u_long ip_address_;
	bool destroy_;
};

inline ActiveUser::
~ActiveUser()
{
	if( conf_ ) 
	{
		Conference* cur_conf = conf_;
		cur_conf->leave( this );
		if( cur_conf->is_closed() )
			delete cur_conf;
	}
	delete user_info_;
	delete chandler_;
	ACE_DEBUG(( LM_DEBUG, "(%t) ActiveUser 0x%x being deleted\n", this ));
}

inline int ActiveUser::
putq( ACE_Message_Block* mb, ACE_Time_Value* tv )
{
	return mqueue_.enqueue_tail( mb, tv );
}

inline u_long ActiveUser::
get_ip_address()
{
	return ip_address_;
}

inline Participant::ParticipantKey ActiveUser::
key()
{
	return ip_address_;
}

inline int ActiveUser::
send( DSMessage* msg )
{
	ACE_DEBUG(( LM_DEBUG, "(%t) (ActiveUser::send) User: 0x%x, msg: 0x%x\n", this, msg ));
	if( msg->send( chandler_->peer() ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ActiveUser::send) msg 0x%x send error, handle %d\n",
						  msg, chandler_->peer().get_handle() ), -1 );

	return 0;
}

#endif
