/*
 * File         : ActiveUserRep.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1998/02/28 00:57:28 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __ACTIVEUSERREP_H )
#define __ACTIVEUSERREP_H

#include "DSHashMapManager.h"
#include <ace/Singleton.h>

class ActiveUser;

typedef u_long ActiveUserKey;
typedef ActiveUser* ActiveUserEnt;
typedef DSHashMapManager<ActiveUserKey, ActiveUserEnt> DSActiveUserRep;
typedef DSHashMapIterator<ActiveUserKey, ActiveUserEnt> DSActiveUserIterator;
typedef ACE_Singleton<DSActiveUserRep, ACE_Null_Mutex> SDSActiveUserRep;

inline u_long ACE_Hash_Map_Manager<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>::
hash( const ActiveUserKey& ext_id )
{
	return ext_id;
}

inline int ACE_Hash_Map_Iterator_Base<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>::
next (ACE_Hash_Map_Entry<ActiveUserKey, ActiveUserEnt> *& entry)
{
	ACE_TRACE ("ACE_Hash_Map_Iterator_Base<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>::next");

  if (this->map_man_->table_ != 0
      && this->index_ < ACE_static_cast (ssize_t, this->map_man_->total_size_)
      && this->index_ >= 0
      && this->next_ != &this->map_man_->table_[this->index_])
    {
      entry = this->next_;
      return 1;
    }
  else
    return 0;
}

inline int ACE_Hash_Map_Iterator_Base<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>::
forward_i (void)
{
	ACE_TRACE ("ACE_Hash_Map_Iterator_Base<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>::forward_i");

  if (this->map_man_->table_ == 0)
    return -1;

  // Guard against error condition first.
  if (this->index_ < 0)
    {
      this->index_++;
      return this->forward_i ();
    }
  else if (this->index_ >= ACE_static_cast(ssize_t, this->map_man_->total_size_))
    return 0;

  this->next_ = this->next_->next_;
  if (this->next_ == &this->map_man_->table_[this->index_])
    {
      while (++this->index_ < ACE_static_cast (ssize_t, this->map_man_->total_size_))
	{
	  this->next_ = &this->map_man_->table_[this->index_];
	  if (this->next_->next_ != &this->map_man_->table_[this->index_])
	    {
	      this->next_ = this->map_man_->table_[this->index_].next_;
	      return 1;
	    }
	}
      return 0;
    }
  return 1;
}

#endif
