/*
 * File         : ClientHandler.cpp
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.5 $
 * Updated      : $Date: 1998/03/13 00:01:14 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include "ClientHandler.h"
#include "ActiveUser.h"
#include "ActiveUserRep.h"
#include "DirectoryProtocol.h"

int ClientHandler::
open( void* )
{
	ACE_TRACE( "ClientHandler::open" );
	ACE_INET_Addr addr;
	if( peer().get_remote_addr( addr ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ClientHandler::open) address error\n" ), -1 );
	else
    {
		ACE_NEW_RETURN( user_, ActiveUser( this ), -1 );
		if( SDSActiveUserRep::instance()->bind( user_->key(), user_ ) != 0 )
			ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ClientHandler::open) can't bind in repository\n" ), -1 );

		if( ACE_Reactor::instance()->register_handler( this, READ_MASK ) == -1 )
			ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ClientHandler::open) can't register with reactor\n" ), -1 );
		
		ACE_DEBUG(( LM_DEBUG, "(%t) %D, [%s] connection opened, handle %d, user: 0x%x\n",
					addr.get_host_name(), peer().get_handle(), user_ ));
	}
	return 0;
}

int ClientHandler::
handle_input( ACE_HANDLE handle )
{
	ACE_TRACE( "ClientHandler::handle_input" );
	ACE_DEBUG(( LM_DEBUG, "(%t) input received on handle %d, user: 0x%x\n",
				handle, user_ ));
	if( SDSProtocol::instance()->recv( peer(), msg_ ) == -1 )
	{
		if( SDSActiveUserRep::instance()->unbind( user_->key() ) == -1 )
			ACE_ERROR(( LM_ERROR, "(%t) [!](ClientHandler::handle_input) unbinding error: 0x%x\n", user_ ));
		
		return -1;
	}
	msg_->user( user_ );
	msg_mb_ = new ACE_Message_Block( ( const char* )msg_, 1 );
	svc_mb_ = new ACE_Message_Block( ( const char* )user_, 1 );
	if( !( msg_mb_ && svc_mb_ ) )
	{
		delete msg_mb_;
		delete svc_mb_;
		delete msg_;
	}
	else
	{
		SDSThreadPool::instance()->putq( svc_mb_ );
		user_->putq( msg_mb_ );
	}
	return 0;
}

int ClientHandler::
handle_close( ACE_HANDLE handle, ACE_Reactor_Mask m )
{
	ACE_TRACE( "ClientHandler::handle_close" );
	ACE_INET_Addr addr;
	peer().get_remote_addr( addr );
	ACE_DEBUG(( LM_DEBUG, "(%t) %D, [%s] connection closed, handle %d\n",
				addr.get_host_name(), handle ));
	shutdown();
	if( ! user_ )
		ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](ClientHandler::handle_close) user not valid\n"), -1 );

	SDSProtocol::instance()->get( DirectoryProtocol::SHUTDOWN, msg_ );
	msg_->user( user_ );
	msg_mb_ = new ACE_Message_Block( ( const char* )msg_, 1 );
	svc_mb_ = new ACE_Message_Block( ( const char* )user_, 1 );
	if( !( msg_mb_ && svc_mb_ ) )
	{
		delete msg_mb_;
		delete svc_mb_;
		delete msg_;
	}
	else
	{
		SDSThreadPool::instance()->putq( svc_mb_ );
		user_->putq( msg_mb_ );
	}
	return 0;
}

#if defined( ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA )
#pragma instantiate ACE_Svc_Handler<ACE_SOCK_STREAM, ACE_NULL_SYNCH>
#pragma instantiate MsgHeader<ACE_SOCK_Stream>
#pragma instantiate Message<DSMsgHeader>
#pragma instantiate MsgFlyweight<DSMessage::MID, DSMessage*>
#pragma instantiate ProtocolInterface<DSMessage, DSMsgFlyweight>
#pragma instantiate ACE_Singleton<DSProtocol, ACE_Null_Mutex>
#pragma instantiate ACE_Singleton<DSMsgFlyweight, ACE_Null_Mutex>
#pragma instantiate ACE_Singleton<ThreadPool, ACE_Null_Mutex>
#pragma instantiate ACE_Singleton<DSActiveUserRep, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Entry<ActiveUserKey, ActiveUserEnt>
#pragma instantiate ACE_Hash_Map_Manager<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator<ActiveUserKey, ActiveUserEnt, ACE_RW_Thread_Mutex>
#endif
