/*
 * File         : ConferenceFactory.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 20 20:06:26 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/03/13 00:01:15 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __CONFERENCEFACTORY_H )
#define __CONFERENCEFACTORY_H

#include "OpenConference.h"
#include "RestrictedConference.h"
#include "ProtectedConference.h"

#include <ace/Singleton.h>

class ConferenceFactory
{
public:
	typedef char ConferenceType;
	enum { OPEN, RESTRICTED, PROTECTED };
	Conference* create_conference( ConferenceType ctype, void* params = 0, size_t size = 0 );
};

typedef ACE_Singleton<ConferenceFactory, ACE_Null_Mutex> SDSConferenceFactory;

inline Conference* ConferenceFactory::
create_conference( ConferenceType ctype, void* params, size_t size )
{
	Conference* new_conf;
	switch( ctype )
	{
	case OPEN:
		new_conf = new OpenConference;
		break;

	case RESTRICTED:
		new_conf = new RestrictedConference;
		break;

	case PROTECTED:
		new_conf = new ProtectedConference;
		break;

	default:
		return 0;
	}
	if( new_conf )
		if( new_conf->open( params, size ) == -1 )
		{
			new_conf->close();
			delete new_conf;
			return 0;
		}

	return new_conf;
}

#endif
