/*
 * File         : ConferenceRep.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 20 14:06:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:30 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __CONFERENCEREP_H )
#define __CONFERENCEREP_H

#include "DSHashMapManager.h"
#include <ace/Singleton.h>

class Conference;

typedef u_long ConferenceKey;
typedef Conference* ConferenceEnt;
typedef DSHashMapManager<ConferenceKey, ConferenceEnt> DSConferenceRep;
typedef DSHashMapIterator<ConferenceKey, ConferenceEnt> DSConferenceIterator;
typedef ACE_Singleton<DSConferenceRep, ACE_Null_Mutex> SDSConferenceRep;

inline u_long ACE_Hash_Map_Manager<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>::
hash( const ConferenceKey& ext_id )
{
	return ext_id;
}

inline int ACE_Hash_Map_Iterator_Base<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>::
next (ACE_Hash_Map_Entry<ConferenceKey, ConferenceEnt> *& entry)
{
	ACE_TRACE ("ACE_Hash_Map_Iterator_Base<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>::next");

  if (this->map_man_->table_ != 0
      && this->index_ < ACE_static_cast (ssize_t, this->map_man_->total_size_)
      && this->index_ >= 0
      && this->next_ != &this->map_man_->table_[this->index_])
    {
      entry = this->next_;
      return 1;
    }
  else
    return 0;
}

inline int ACE_Hash_Map_Iterator_Base<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>::
forward_i (void)
{
	ACE_TRACE ("ACE_Hash_Map_Iterator_Base<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>::forward_i");

  if (this->map_man_->table_ == 0)
    return -1;

  // Guard against error condition first.
  if (this->index_ < 0)
    {
      this->index_++;
      return this->forward_i ();
    }
  else if (this->index_ >= ACE_static_cast(ssize_t, this->map_man_->total_size_))
    return 0;

  this->next_ = this->next_->next_;
  if (this->next_ == &this->map_man_->table_[this->index_])
    {
      while (++this->index_ < ACE_static_cast (ssize_t, this->map_man_->total_size_))
	{
	  this->next_ = &this->map_man_->table_[this->index_];
	  if (this->next_->next_ != &this->map_man_->table_[this->index_])
	    {
	      this->next_ = this->map_man_->table_[this->index_].next_;
	      return 1;
	    }
	}
      return 0;
    }
  return 1;
}

#endif
