/*
 * File         : DirectoryService.cpp
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.5 $
 * Updated      : $Date: 1998/03/13 00:01:18 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include "DirectoryService.h"
#include "DirectoryProtocol.h"
#include "DSSingletons.h"
#include "ActiveUserRep.h"
#include "ConferenceRep.h"
#include "msgINIT.h"
#include "msgREGISTER.h"
#include "msgUNREG.h"
#include "msgUSYNCH.h"
#include "msgINVITE.h"
#include "msgUSERIP.h"
#include "msgNEWCONF.h"
#include "msgCSYNCH.h"
#include "msgDELCONF.h"
#include "msgJOIN.h"
#include "msgLEAVE.h"
#include "msgINFO.h"
#include "msgPASSWD.h"
#include "msgACCEPT.h"
#include "msgSHUTDOWN.h"

#include <ace/Get_Opt.h>

int DirectoryService::
init( int argc, char* argv[] )
{
	ACE_TRACE( "(DirectoryService::init)" );
	if( parse_args( argc, argv ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) parsing error\n" ), -1 );

	if( acceptor_.open( ACE_INET_Addr( port_ ), ACE_Reactor::instance() ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) %p\n", "open" ), -1 );

	DSMsgFlyweight* msg_pool = SDSMsgFlyweight::instance();
	if( msg_pool->open( DirectoryProtocol::MSGMAX ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) msg_pool_ error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::INIT, new msgINIT<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::REGISTER, new msgREGISTER<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::UNREG, new msgUNREG<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::USYNCH, new msgUSYNCH<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::INVITE, new msgINVITE<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::USERIP, new msgUSERIP<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::NEWCONF, new msgNEWCONF<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::CSYNCH, new msgCSYNCH<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::DELCONF, new msgDELCONF<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::JOIN, new msgJOIN<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::LEAVE, new msgLEAVE<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::INFO, new msgINFO<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::PASSWD, new msgPASSWD<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::ACCEPT, new msgACCEPT<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( msg_pool->add( DirectoryProtocol::SHUTDOWN, new msgSHUTDOWN<DSMsgHeader> ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Message adding error\n" ), -1 );

	if( SDSProtocol::instance()->open( msg_pool ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Protocol error\n" ), -1 );

	if( SDSActiveUserRep::instance()->open() == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Active User repository error\n" ), -1 );

	if( SDSConferenceRep::instance()->open() == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) Conference repository error\n" ), -1 );

	if( SDSThreadPool::instance()->open( threads_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](DirectoryService::init) thread_pool_ error\n" ), -1 );

	ACE_DEBUG(( LM_DEBUG, "Directory Service initialized [port: %d, threads: %d]\n",
		port_, threads_ ));
	return 0;
}

int DirectoryService::
fini()
{
	ACE_TRACE( "(DirectoryService::fini)" );
	ACE_DEBUG(( LM_DEBUG, "Directory Service shutdown\n" ));
	return 0;
}

int DirectoryService::
parse_args( int argc, char* argv [] )
{
	ACE_TRACE( "(DirectoryService::parse_args)" );
	ACE_Get_Opt get_opt( argc, argv, "p:t:" );
	int c;
	while( ( c = get_opt() ) != -1 )
		switch( c )
		{
		case 'p':
			port_ = ( unsigned short )ACE_OS::atoi( get_opt.optarg );
			break;
		case 't':
			threads_ = ( unsigned short )ACE_OS::atoi( get_opt.optarg );
			break;
		default:
			return -1;
		}
	return 0;
}

// Define the factory function.
ACE_SVC_FACTORY_DEFINE(DirectoryService)

// Define the object that describes the service.
ACE_STATIC_SVC_DEFINE (DirectoryService,
		       "DirectoryService", ACE_SVC_OBJ_T, &ACE_SVC_NAME (DirectoryService),
		       ACE_Service_Type::DELETE_THIS, 0)

#if defined( ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA )
#pragma instantiate ACE_Acceptor<ClientHandler, ACE_SOCK_ACCEPTOR>
#pragma instantiate msgINIT<DSMsgHeader>
#pragma instantiate msgREGISTER<DSMsgHeader>
#pragma instantiate msgUNREG<DSMsgHeader>
#pragma instantiate msgUSYNCH<DSMsgHeader>
#pragma instantiate msgINVITE<DSMsgHeader>
#pragma instantiate msgUSERIP<DSMsgHeader>
#pragma instantiate msgNEWCONF<DSMsgHeader>
#pragma instantiate msgCSYNCH<DSMsgHeader>
#pragma instantiate msgDELCONF<DSMsgHeader>
#pragma instantiate msgJOIN<DSMsgHeader>
#pragma instantiate msgLEAVE<DSMsgHeader>
#pragma instantiate msgINFO<DSMsgHeader>
#pragma instantiate msgPASSWD<DSMsgHeader>
#pragma instantiate msgACCEPT<DSMsgHeader>
#pragma instantiate DSHashMapManager<ConferenceKey, ConferenceEnt>
#pragma instantiate DSHashMapIterator<ConferenceKey, ConferenceEnt>
#pragma instantiate ACE_Singleton<DSConferenceRep, ACE_Null_Mutex>
#pragma instantiate auto_ptr<DSMessage>
#pragma instantiate ACE_Auto_Basic_Ptr<DSMessage>
#pragma instantiate ACE_Hash_Map_Entry<ConferenceKey, ConferenceEnt>
#pragma instantiate ACE_Hash_Map_Manager<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator<ConferenceKey, ConferenceEnt, ACE_RW_Thread_Mutex>
#pragma instantiate ACE_Singleton<ConferenceFactory, ACE_Null_Mutex>
#endif
