/*
 * File         : MsgFlyweight.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/02/28 00:57:34 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGFLYWEIGHT_H )
#define __MSGFLYWEIGHT_H

template<class KEY_TYPE, class ITEM_TYPE>
class MsgFlyweight
{
public:
      typedef KEY_TYPE idx_type;
      typedef ITEM_TYPE value_type;

      ~MsgFlyweight();
	  int open( const KEY_TYPE& a_size );
      int add( const KEY_TYPE& idx, const ITEM_TYPE& item );
      int remove( const KEY_TYPE& idx );
	  int get( const KEY_TYPE& idx, ITEM_TYPE& item );
      
private:
      ITEM_TYPE* store_ptr_;
	  char* fill_map_;
      KEY_TYPE size_;
};

template<class KEY_TYPE, class ITEM_TYPE>
int MsgFlyweight<KEY_TYPE, ITEM_TYPE>::
open( const KEY_TYPE& a_size ) 
{
	size_ = a_size;
	if( !( store_ptr_ = new ITEM_TYPE[size_] )  )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgFlyweight::open) No memory for store_ptr_\n" ),
		  -1);
	
	if( !( fill_map_ = new char[size_] ) )
      ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgFlyweight::open) No memory for fill_map_\n" ),
		  -1);
   
	for( char i=0; i<size_; i++ )
      fill_map_[i] = 0;
   
   return 0;
}

// If pointers are stored, deletion of the objects 
// should occur before this desctructor
template<class KEY_TYPE, class ITEM_TYPE>
MsgFlyweight<KEY_TYPE, ITEM_TYPE>::
~MsgFlyweight() 
{
	delete[] store_ptr_;
	delete[] fill_map_;
}

template<class KEY_TYPE, class ITEM_TYPE>
int MsgFlyweight<KEY_TYPE, ITEM_TYPE>::
add( const KEY_TYPE& idx, const ITEM_TYPE& item )
{
	if( idx < size_ )
	{
		if( fill_map_[idx] )
			ACE_ERROR_RETURN(( LM_ERROR,
				"[!](MsgFlyweight::add) overwritting previous item %d\n", idx ), -1 );

		store_ptr_[idx] = item;
		fill_map_[idx] = 1;
	}
	else
		ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgFlyweight::add) No room for %d\n", idx ), -1 );

	return 0;
}

template<class KEY_TYPE, class ITEM_TYPE>
int MsgFlyweight<KEY_TYPE, ITEM_TYPE>::
remove( const KEY_TYPE& idx )
{
   if( idx < size_ )
      fill_map_[idx] = 0;
   else
      ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgFlyweight::remove) %d: does not exist\n", idx ),
						-1 );
   return 0;
}

template<class KEY_TYPE, class ITEM_TYPE>
int MsgFlyweight<KEY_TYPE, ITEM_TYPE>::
get( const KEY_TYPE& idx, ITEM_TYPE& item )
{
	if( ( idx < size_ )&&( fill_map_[idx] ) )
		item = store_ptr_[idx];
	else
		ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgFlyweight::get) %d: does not exist\n", idx ),
							-1 );
	return 0;
}

#endif
