/*
 * File         : MsgHeader.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/02/28 00:57:35 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGHEADER_H)
#define __MSGHEADER_H

template<class SAP_TYPE>
class MsgHeader
{
public:
	typedef SAP_TYPE SAP;
	typedef unsigned short MLEN;
	typedef unsigned char MID;
	enum { HLEN = 2+1 };

	MsgHeader( const MID& mid = 0, const MLEN& mlen = 0 );
	int recv( SAP_TYPE& sap );
	int send( SAP_TYPE& sap );
	const MLEN& msg_length() const;
	void msg_length( const MLEN& mlen );
	const MID& msgID() const;
	int size() { return HLEN; }

private:
	char header_[HLEN];
};

template<class SAP_TYPE>
inline MsgHeader<SAP_TYPE>::
MsgHeader( const MID& mid, const MLEN& mlen )
{
	*( MLEN* )header_ = mlen;
	*( MID* )( header_ + sizeof( MLEN ) ) = mid;
}

template<class SAP_TYPE>
inline int MsgHeader<SAP_TYPE>::
recv( SAP_TYPE& sap )
{
	if( sap.recv_n( header_, sizeof header_ ) != sizeof header_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgHeader<SAP_TYPE>::recv)\n" ), -1 );

	*( MLEN* )header_ = ntohs( *( MLEN* )header_ );

	return 0;
}

template<class SAP_TYPE>
inline int MsgHeader<SAP_TYPE>::
send( SAP_TYPE& sap )
{
	*( MLEN* )header_ = htons( *( MLEN* )header_ );

	if( sap.send_n( header_, sizeof header_ ) != sizeof header_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](MsgHeader<SAP_TYPE>::send)\n" ), -1 );

	*( MLEN* )header_ = ntohs( *( MLEN* )header_ );

	return 0;
}

template<class SAP_TYPE>
inline const MsgHeader<SAP_TYPE>::MLEN& MsgHeader<SAP_TYPE>::
msg_length() const
{
	return *( MLEN* )header_;
}

template<class SAP_TYPE>
inline void MsgHeader<SAP_TYPE>::
msg_length( const MLEN& mlen )
{
	*( MLEN* )header_ = mlen;
}

template<class SAP_TYPE>
inline const MsgHeader<SAP_TYPE>::MID& MsgHeader<SAP_TYPE>::
msgID() const
{
	return *( MID* )( header_ + sizeof( MLEN ) );
}

#endif
