/*
 * File         : OpenConference.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 20 14:06:26 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/03/13 00:01:19 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __OPENCONFERENCE_H )
#define __OPENCONFERENCE_H

#include "Conference.h"
#include "ParticipantRep.h"

#include <algorithm>

#include <ace/Synch_T.h>

class OpenConference
: public Conference
{
public:
	~OpenConference();
	int open( void* = 0, size_t = 0 ) { return 0; }
	int close( void* = 0, size_t = 0 ) { return 0; }
	int join( Participant* part );
	int leave( Participant* part );
	int set_chair( Participant* part );
	int build_key_vector( KeyVector keys );
	unsigned int cur_num_part();
	unsigned int max_num_part() { return 0; }

private:
	DSParticipantRep participants_;
	ACE_RW_Thread_Mutex lock_;
};


inline unsigned int OpenConference::
cur_num_part()
{
	ACE_READ_GUARD_RETURN( ACE_RW_Thread_Mutex, mon, lock_, -1 );
	return participants_.size();
}

#endif
