/*
 * File         : ProtocolInterface.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/02/28 00:57:36 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __PROTOCOLINTERFACE_H )
#define __PROTOCOLINTERFACE_H

template<class MSG, class CONT>
class ProtocolInterface
{
public:
	typedef MSG Message;
	typedef MSG::Header Header;
	typedef MSG::SAP SAP;
	typedef MSG::MID MID;

	int open( CONT* cont ) { cont_ = cont ; return 0; }
	int recv( SAP&, MSG*& );
	int send( SAP&, MSG* );
	int get( const MID&, MSG*& );
	
protected:
	Header header_;
	CONT* cont_;
};

template<class MSG, class CONT>
int ProtocolInterface<MSG, CONT>::
recv( SAP& sap, MSG*& msg )
{
	if( header_.recv( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface<MSG, CONT>::recv) header\n" ), -1 );

	if( cont_->get( header_.msgID(), msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface<MSG, CONT>::recv) get\n" ), -1 );
		
	if( !( msg = msg->clone() ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface<MSG, CONT>::recv) clone\n" ), -1 );

	if( msg->recv( sap, header_ ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface<MSG, CONT>::recv) recv\n" ), -1 );

	return 0;
}

template<class MSG, class CONT>
inline int ProtocolInterface<MSG, CONT>::
send( SAP& sap, MSG* msg )
{
	if( msg->send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface<MSG, CONT>::send) send\n" ), -1 );

	return 0;
}

template<class MSG, class CONT>
inline int ProtocolInterface<MSG, CONT>::
get( const MID& mid, MSG*& msg )
{
	if( cont_->get( mid, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface<MSG, CONT>::get) get\n" ), -1 );

	if( !( msg = msg->clone() ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](ProtocolInterface<MSG, CONT>::get) clone\n" ), -1 );

	return 0;
}

#endif
