/*
 * File         : msgACCEPT.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 26 11:00:26 1998
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1998/03/17 22:01:18 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGACCEPT_H )
#define __MSGACCEPT_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "RestrictedConference.h"

template<class HEADER>
class msgACCEPT
: public Message<HEADER>
{
public:
	msgACCEPT();
	~msgACCEPT();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgACCEPT<HEADER>; }

protected:
	ConferenceKey key_[2];
};

template<class HEADER>
inline msgACCEPT<HEADER>::
msgACCEPT()
: Message<HEADER>( HEADER( DirectoryProtocol::ACCEPT, sizeof key_ ) )
{
	key_[0] = 0;
	key_[1] = 0;
}

template<class HEADER>
msgACCEPT<HEADER>::
~msgACCEPT() 
{
}

template<class HEADER>
int msgACCEPT<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgACCEPT<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgACCEPT<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::set) setting error\n" ), -1 );

	memcpy( key_, buf, sizeof key_ );
	return 0;
}

template<class HEADER>
int msgACCEPT<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing ACCEPT msg 0x%x, user: 0x%x\n", this, user_ ));
	ActiveUser* user;
	DSActiveUserIterator uit( *SDSActiveUserRep::instance() );
	uit.acquire_read();
	if( SDSActiveUserRep::instance()->find( key_[1], user ) == -1 )
	{
		uit.release();
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::execute) user %d not found\n", key_[1] ), -1 );
	}
	if( ! key_[0] )
	{
		user->send_msg( DirectoryProtocol::INFO, "Your request has been rejected" );
		uit.release();
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::execute) Participant 0x%x refused to join conf\n", user ), -1 );
	}
	Conference* conf;
	DSConferenceIterator cit( *SDSConferenceRep::instance() );
	cit.acquire_read();
	if( SDSConferenceRep::instance()->find( key_[0], conf ) == -1 )
	{
		cit.release();
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgACCEPT<HEADER>::execute) conf %d not found\n", key_[0] ), -1 );
	}
	( ( RestrictedConference* )conf )->accept( user );
	uit.release();
	cit.release();
	return 0;
}

#endif
