/*
 * File         : msgCSYNCH.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 23 11:06:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:38 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGCSYNCH_H)
#define __MSGCSYNCH_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ConferenceFactory.h"

template<class HEADER>
class msgCSYNCH
: public Message<HEADER>
{
public:
	msgCSYNCH();
	~msgCSYNCH();
	int recv( SAP& sap, HEADER& header );
	int send( SAP& sap );
	int set( void* params, size_t len = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgCSYNCH<HEADER>; }

private:
	char* buf_;
};

template<class HEADER>
inline msgCSYNCH<HEADER>::
msgCSYNCH()
: Message<HEADER>( HEADER( DirectoryProtocol::CSYNCH ) ), buf_( 0 )
{
}

template<class HEADER>
msgCSYNCH<HEADER>::
~msgCSYNCH() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgCSYNCH<HEADER>::
recv( SAP& sap, HEADER& header )
{
	delete[] buf_;
	if( !( buf_ = new char[header.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCSYNCH<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header.msg_length() ) != header.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCSYNCH<HEADER>::recv) recv error\n" ), -1 );

	header_ = header;
	return 0;
}

template<class HEADER>
int msgCSYNCH<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCSYNCH<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCSYNCH<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgCSYNCH<HEADER>::
set( void* buf, size_t )
{
	ACE_ERROR_RETURN(( LM_ERROR, "[!](msgCSYNCH<HEADER>::set) setting error\n" ), -1 );
}

template<class HEADER>
int msgCSYNCH<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing CSYNCH msg 0x%x, user: 0x%x\n", this, user_ ));
	DSConferenceIterator it( *SDSConferenceRep::instance() );
	DSConferenceIterator end( *SDSConferenceRep::instance(), 1 );
	int len;
	while( it != end )
	{
		delete[] buf_;
		it.acquire_read();
		len = ( *it ).int_id_->build_key_vector( ( Conference::KeyVector )buf_ );
		if( len )
		{
			header_.msg_length( len );
			user_->send( this );
		}
		it++;
		it.release();
	}
	return 0;
}

#endif
