/*
 * File         : msgDELCONF.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 24 11:00:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:39 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGDELCONF_H )
#define __MSGDELCONF_H

#include "DirectoryProtocol.h"
#include "Message.h"

template<class HEADER>
class msgDELCONF
: public Message<HEADER>
{
public:
	msgDELCONF();
	~msgDELCONF();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgDELCONF<HEADER>; }

protected:
	ConferenceKey key_;
};

template<class HEADER>
inline msgDELCONF<HEADER>::
msgDELCONF()
: Message<HEADER>( HEADER( DirectoryProtocol::DELCONF, sizeof key_ ) ), key_( 0 )
{
}

template<class HEADER>
msgDELCONF<HEADER>::
~msgDELCONF() 
{
}

template<class HEADER>
int msgDELCONF<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgDELCONF<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgDELCONF<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgDELCONF<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgDELCONF<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgDELCONF<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgDELCONF<HEADER>::set) setting error\n" ), -1 );

	key_ = *( ConferenceKey* )buf;
	return 0;
}

template<class HEADER>
int msgDELCONF<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing DELCONF msg 0x%x, user: 0x%x\n",
				this, user_ ));
	return 0;
}

#endif
