/*
 * File         : msgINFO.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 26 13:00:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:39 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGINFO_H )
#define __MSGINFO_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ActiveUser.h"
#include "UserInfo.h"

template<class HEADER>
class msgINFO
: public Message<HEADER>
{
public:
	msgINFO();
	~msgINFO();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgINFO<HEADER>; }

protected:
	char* buf_;
};

template<class HEADER>
inline msgINFO<HEADER>::
msgINFO()
: Message<HEADER>( HEADER( DirectoryProtocol::INFO ) ), buf_( 0 )
{
}

template<class HEADER>
msgINFO<HEADER>::
~msgINFO() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgINFO<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	delete[] buf_;
	if( !( buf_ = new char[header_.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINFO<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINFO<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgINFO<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINFO<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINFO<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgINFO<HEADER>::
set( void* buf, size_t len )
{
	delete[] buf_;
	if( !( buf_ = strdup( ( const char* )buf ) ) )
	{
		header_.msg_length( 0 );	
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINFO<HEADER>::set) allocation error\n" ), -1 );
	}
	header_.msg_length( strlen( buf_ ) + 1 );
	return 0;
}

template<class HEADER>
int msgINFO<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing INFO msg 0x%x, user: 0x%x, key: %u\n",
				this, user_, user_->key() ));
	return 0;
}

#endif
