/*
 * File         : msgINIT.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/02/28 00:57:40 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGINIT_H)
#define __MSGINIT_H

#include "DirectoryProtocol.h"
#include "Message.h"


template<class HEADER>
class msgINIT
: public Message<HEADER>
{
public:
	msgINIT();
	~msgINIT();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgINIT<HEADER>; }

private:
	char* buf_;
};

template<class HEADER>
inline msgINIT<HEADER>::
msgINIT()
: Message<HEADER>( HEADER( DirectoryProtocol::INIT ) ), buf_( 0 )
{
}

template<class HEADER>
msgINIT<HEADER>::
~msgINIT() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgINIT<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	delete buf_;
	if( !( buf_ = new char[header_.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINIT<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINIT<HEADER>::recv) recv error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgINIT<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINIT<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINIT<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgINIT<HEADER>::
set( void* buf, size_t )
{
	delete[] buf_;
	if( !( buf_ = strdup( ( char* )buf ) ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINIT<HEADER>::set) setting error\n" ), -1 );
	
	header_.msg_length( strlen( buf_ ) + 1 );
	return 0;
}

template<class HEADER>
int msgINIT<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing INIT msg 0x%x: %s, user: 0x%x\n",
				this, buf_, user_ ));
	return 0;
}

#endif
