/*
 * File         : msgINVITE.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 10 10:38:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:40 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGINVITE_H )
#define __MSGINVITE_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ActiveUser.h"
#include "UserInfo.h"

#include <ace/Auto_Ptr.h>

template<class HEADER>
class msgINVITE
: public Message<HEADER>
{
public:
	msgINVITE();
	~msgINVITE();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgINVITE<HEADER>; }

protected:
	ActiveUserKey key_;
};

template<class HEADER>
inline msgINVITE<HEADER>::
msgINVITE()
: Message<HEADER>( HEADER( DirectoryProtocol::INVITE, sizeof key_ ) ), key_( 0 )
{
}

template<class HEADER>
msgINVITE<HEADER>::
~msgINVITE() 
{
}

template<class HEADER>
int msgINVITE<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINVITE<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgINVITE<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINVITE<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINVITE<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgINVITE<HEADER>::
set( void* buf, size_t len )
{
	if( len != 0 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINVITE<HEADER>::set) setting error\n" ), -1 );

	key_ = *( ActiveUserKey* )buf;
	return 0;
}

template<class HEADER>
int msgINVITE<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing INVITE msg 0x%x, user: 0x%x\n", this, user_ ));
	DSMessage* msg;	
	if( SDSProtocol::instance()->get( DirectoryProtocol::USERIP, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINVITE<HEADER>::execute) protocol error\n" ), -1 );

	auto_ptr<DSMessage> auto_msg( msg );
	ActiveUser* invited_user;
	DSActiveUserIterator it( *SDSActiveUserRep::instance() );
	it.acquire_read();
	if( SDSActiveUserRep::instance()->find( key_, invited_user ) == -1 )
	{
		it.release();
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINVITE<HEADER>::execute) user %d not found\n", key_ ), -1 );
	}
	u_long ip = invited_user->get_ip_address();
	it.release();
	auto_msg->user( user_ );
	auto_msg->set( &ip );
	if( user_->send( auto_msg.get() ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgINVITE<HEADER>::execute) msg send error, user: 0x%x\n", user_ ), -1 );

	return 0;
}

#endif
