/*
 * File         : msgLEAVE.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 26 13:00:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:41 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGLEAVE_H )
#define __MSGLEAVE_H

#include "DirectoryProtocol.h"
#include "Message.h"

template<class HEADER>
class msgLEAVE
: public Message<HEADER>
{
public:
	msgLEAVE();
	~msgLEAVE();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgLEAVE<HEADER>; }

protected:
	ConferenceKey key_[2];
};

template<class HEADER>
inline msgLEAVE<HEADER>::
msgLEAVE()
: Message<HEADER>( HEADER( DirectoryProtocol::LEAVE, sizeof key_ ) )
{
	key_[0] = 0;
	key_[1] = 0;
}

template<class HEADER>
msgLEAVE<HEADER>::
~msgLEAVE() 
{
}

template<class HEADER>
int msgLEAVE<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgLEAVE<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgLEAVE<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgLEAVE<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgLEAVE<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgLEAVE<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgLEAVE<HEADER>::set) setting error\n" ), -1 );

	memcpy( key_, buf, sizeof key_ );
	return 0;
}

template<class HEADER>
int msgLEAVE<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing LEAVE msg 0x%x, user: 0x%x\n",
				this, user_ ));

	Conference* conf;
	DSConferenceIterator it( *SDSConferenceRep::instance() );
	it.acquire_read();
	if( SDSConferenceRep::instance()->find( key_[0], conf ) == -1 )
	{
		it.release();
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgLEAVE<HEADER>::execute) conf %d not found\n", key_[0] ), -1 );
	}
	conf->leave( user_ );
	it.release();
	if( conf->is_closed() )
		delete conf;

	return 0;
}

#endif
