/*
 * File         : msgNEWCONF.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 20 18:06:26 1998
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1998/03/17 22:01:18 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGNEWCONF_H)
#define __MSGNEWCONF_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ConferenceFactory.h"

template<class HEADER>
class msgNEWCONF
: public Message<HEADER>
{
public:
	msgNEWCONF();
	~msgNEWCONF();
	int recv( SAP& sap, HEADER& header );
	int send( SAP& sap );
	int set( void* params, size_t len = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgNEWCONF<HEADER>; }

private:
	char* buf_;
};

template<class HEADER>
inline msgNEWCONF<HEADER>::
msgNEWCONF()
: Message<HEADER>( HEADER( DirectoryProtocol::NEWCONF ) ), buf_( 0 )
{
}

template<class HEADER>
msgNEWCONF<HEADER>::
~msgNEWCONF() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgNEWCONF<HEADER>::
recv( SAP& sap, HEADER& header )
{
	delete[] buf_;
	if( !( buf_ = new char[header.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgNEWCONF<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header.msg_length() ) != header.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgNEWCONF<HEADER>::recv) recv error\n" ), -1 );

	header_ = header;
	return 0;
}

template<class HEADER>
int msgNEWCONF<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgNEWCONF<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgNEWCONF<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgNEWCONF<HEADER>::
set( void* buf, size_t )
{
	ACE_ERROR_RETURN(( LM_ERROR, "[!](msgNEWCONF<HEADER>::set) setting error\n" ), -1 );
}

template<class HEADER>
int msgNEWCONF<HEADER>::
execute()
{
	const char* ptopic = buf_ + sizeof( ConferenceFactory::ConferenceType )
							  + sizeof( ConferenceKey )
							  + sizeof( ActiveUserKey );

	ACE_DEBUG(( LM_DEBUG, "(%t) executing NEWCONF msg 0x%x: %s, user: 0x%x\n",
				this, ptopic, user_ ));

	char* pparams = ( char* )ptopic + strlen( ptopic ) + 1;
	size_t param_size = 0;
	if( ( pparams - buf_ ) >= header_.msg_length() )
		pparams = 0;
	else
		param_size = buf_ + header_.msg_length() - pparams;

	if( user_->conf() )
	{
		user_->send_msg( DirectoryProtocol::INFO, "You should leave you current conference first" );
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgNEWCONF<HEADER>::execute) User 0x%x should leave conf: 0x%x first\n",
						   user_, user_->conf() ), -1 );
	}
	Conference* new_conf = SDSConferenceFactory::instance()->create_conference( ( ConferenceFactory::ConferenceType )buf_[0], pparams, param_size );
	if( ! new_conf )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgNEWCONF<HEADER>::execute) conference creating error\n" ), -1 );

	if( SDSConferenceRep::instance()->bind( new_conf->key(), new_conf ) != 0 )
	{
		delete new_conf;	
		ACE_ERROR_RETURN(( LM_ERROR, "(%t) [!](msgNEWCONF<HEADER>::execute) can't bind in repository\n" ), -1 );
	}

	new_conf->set_chair( user_ );
	new_conf->set_topic( ptopic );
	ACE_DEBUG(( LM_DEBUG, "(%t) new conference: 0x%x\n", new_conf ));
	ConferenceKey conf_key = new_conf->key();
	memcpy( buf_ + sizeof( ConferenceFactory::ConferenceType ), &conf_key, sizeof conf_key );
	ActiveUserKey user_key = user_->key();
	memcpy( buf_ + sizeof( ConferenceFactory::ConferenceType ) + sizeof( ConferenceKey ), &user_key, sizeof user_key );
	user_->bcast( this );
	return 0;
}

#endif
