/*
 * File         : msgREGISTER.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 29 18:00:26 1998
 * Revision     : $Revision: 1.3 $
 * Updated      : $Date: 1998/02/28 00:57:41 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGREGISTER_H )
#define __MSGREGISTER_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ActiveUser.h"
#include "UserInfo.h"

template<class HEADER>
class msgREGISTER
: public Message<HEADER>
{
public:
	msgREGISTER();
	~msgREGISTER();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgREGISTER<HEADER>; }

protected:
	char* buf_;
};

template<class HEADER>
inline msgREGISTER<HEADER>::
msgREGISTER()
: Message<HEADER>( HEADER( DirectoryProtocol::REGISTER ) ), buf_( 0 )
{
}

template<class HEADER>
msgREGISTER<HEADER>::
~msgREGISTER() 
{
	delete[] buf_; 
}

template<class HEADER>
int msgREGISTER<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	delete[] buf_;
	if( !( buf_ = new char[header_.msg_length()] ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgREGISTER<HEADER>::recv) allocation error\n" ), -1 );

	if( sap.recv_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgREGISTER<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgREGISTER<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgREGISTER<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( buf_, header_.msg_length() ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgREGISTER<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgREGISTER<HEADER>::
set( void* buf, size_t len )
{
	if( len != 4 * sizeof( const char* ) )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgREGISTER<HEADER>::set) setting error\n" ), -1 );

	ActiveUserKey key = **( ActiveUserKey** )buf;
	const char** info = ( const char** )buf + 1;
	const size_t info0_len = strlen( info[0] ) + 1;
	const size_t info1_len = strlen( info[1] ) + 1;
	const size_t all_info_len =  sizeof key + 
								 info0_len +
								 info1_len +
								 strlen( info[2] ) +
								 1;
	delete[] buf_;
	ACE_NEW_RETURN( buf_, char[ all_info_len ], -1 );
	memcpy( buf_, &key, sizeof key );
	strcpy( buf_ + sizeof key, info[0] );
	strcpy( buf_ + sizeof key + info0_len, info[1] );
	strcpy( buf_ + sizeof key + info0_len + info1_len, info[2] );
	header_.msg_length( all_info_len );
	return 0;
}

template<class HEADER>
int msgREGISTER<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing REGISTER msg 0x%x, user: 0x%x, key: %u\n",
				this, user_, user_->key() ));
	ActiveUserKey* p_key = ( ActiveUserKey* )buf_;
	const char* fn = buf_ + sizeof( ActiveUserKey );
	const char* ln = fn + strlen( fn ) + 1;
	const char* em = ln + strlen( ln ) + 1;
	UserInfo* ui;
	ACE_NEW_RETURN( ui, UserInfo( fn, ln, em ), -1 );
	user_->info( ui );
	*p_key = user_->key();
	user_->bcast( this );
	return 0;
}

#endif
