/*
 * File         : msgSHUTDOWN.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Jan 28 18:06:26 1998
 * Revision     : $Revision: 1.4 $
 * Updated      : $Date: 1998/02/28 00:57:42 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined(__MSGSHUTDOWN_H)
#define __MSGSHUTDOWN_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ActiveUser.h"
#include "DSSingletons.h"

template<class HEADER>
class msgSHUTDOWN
: public Message<HEADER>
{
public:
	msgSHUTDOWN();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t = 0 );
	int execute();
	Message<HEADER>* clone() { return new msgSHUTDOWN<HEADER>; }
};

template<class HEADER>
inline msgSHUTDOWN<HEADER>::
msgSHUTDOWN()
: Message<HEADER>( HEADER( DirectoryProtocol::SHUTDOWN ) )
{
}

template<class HEADER>
inline int msgSHUTDOWN<HEADER>::
recv( SAP&, HEADER& )
{
	return 0;
}

template<class HEADER>
inline int msgSHUTDOWN<HEADER>::
send( SAP& )
{
	return 0;
}

template<class HEADER>
inline int msgSHUTDOWN<HEADER>::
set( void*, size_t )
{
	return 0;
}

template<class HEADER>
inline int msgSHUTDOWN<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing SHUTDOWN msg 0x%x, user: 0x%x\n",
				this, user_ ));

	ActiveUserKey key = user_->key();
	user_->bcast_msg( DirectoryProtocol::UNREG, &key, sizeof key );
	ACE_DEBUG(( LM_DEBUG, "(%t) scheduling destruction of ActiveUser: 0x%x\n", user_ ));
	user_->schedule_destruction();
	return 0;
}

#endif
