/*
 * File         : msgUNREG.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 09 11:00:26 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/02/28 00:57:42 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGUNREG_H )
#define __MSGUNREG_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ActiveUser.h"
#include "UserInfo.h"

template<class HEADER>
class msgUNREG
: public Message<HEADER>
{
public:
	msgUNREG();
	~msgUNREG();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgUNREG<HEADER>; }

protected:
	ActiveUserKey key_;
};

template<class HEADER>
inline msgUNREG<HEADER>::
msgUNREG()
: Message<HEADER>( HEADER( DirectoryProtocol::UNREG, sizeof key_ ) ), key_( 0 )
{
}

template<class HEADER>
msgUNREG<HEADER>::
~msgUNREG() 
{
}

template<class HEADER>
int msgUNREG<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUNREG<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgUNREG<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUNREG<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &key_, sizeof key_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUNREG<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgUNREG<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof key_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUNREG<HEADER>::set) setting error\n" ), -1 );

	key_ = *( ActiveUserKey* )buf;
	return 0;
}

template<class HEADER>
int msgUNREG<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing UNREG msg 0x%x, user: 0x%x\n",
				this, user_ ));
	return 0;
}

#endif
