/*
 * File         : msgUSERIP.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 10 10:38:26 1998
 * Revision     : $Revision: 1.1 $
 * Updated      : $Date: 1998/02/28 00:57:42 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGUSERIP_H )
#define __MSGUSERIP_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ActiveUser.h"
#include "UserInfo.h"

template<class HEADER>
class msgUSERIP
: public Message<HEADER>
{
public:
	msgUSERIP();
	~msgUSERIP();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgUSERIP<HEADER>; }

protected:
	u_long ip_;
};

template<class HEADER>
inline msgUSERIP<HEADER>::
msgUSERIP()
: Message<HEADER>( HEADER( DirectoryProtocol::USERIP, sizeof ip_ ) ), ip_( 0 )
{
}

template<class HEADER>
msgUSERIP<HEADER>::
~msgUSERIP() 
{
}

template<class HEADER>
int msgUSERIP<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	if( sap.recv_n( &ip_, sizeof ip_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUSERIP<HEADER>::recv) recv error\n" ), -1 );
	
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgUSERIP<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUSERIP<HEADER>::send) header send error\n" ), -1 );

	if( sap.send_n( &ip_, sizeof ip_ ) != header_.msg_length() )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUSERIP<HEADER>::send) body send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgUSERIP<HEADER>::
set( void* buf, size_t len )
{
	if( len != sizeof ip_ )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUSERIP<HEADER>::set) setting error\n" ), -1 );

	ip_ = *( u_long* )buf;
	return 0;
}

template<class HEADER>
int msgUSERIP<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing USERIP msg 0x%x, user: 0x%x\n",
				this, user_ ));
	return 0;
}

#endif
