/*
 * File         : msgUSYNCH.h
 * Author       : Pawel Roman (proman@npac.syr.edu)
 * Created      : Wed Feb 09 14:00:26 1998
 * Revision     : $Revision: 1.2 $
 * Updated      : $Date: 1998/02/28 00:57:44 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#if !defined( __MSGUSYNCH_H )
#define __MSGUSYNCH_H

#include "DirectoryProtocol.h"
#include "Message.h"
#include "ActiveUser.h"
#include "UserInfo.h"

template<class HEADER>
class msgUSYNCH
: public Message<HEADER>
{
public:
	msgUSYNCH();
	~msgUSYNCH();
	int recv( SAP&, HEADER& );
	int send( SAP& );
	int set( void*, size_t len );
	int execute();
	Message<HEADER>* clone() { return new msgUSYNCH<HEADER>; }
};

template<class HEADER>
inline msgUSYNCH<HEADER>::
msgUSYNCH()
: Message<HEADER>( HEADER( DirectoryProtocol::USYNCH ) )
{
}

template<class HEADER>
msgUSYNCH<HEADER>::
~msgUSYNCH() 
{
}

template<class HEADER>
int msgUSYNCH<HEADER>::
recv( SAP& sap, HEADER& header_ )
{
	this->header_ = header_;
	return 0;
}

template<class HEADER>
int msgUSYNCH<HEADER>::
send( SAP& sap )
{
	if( header_.send( sap ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUSYNCH<HEADER>::send) header send error\n" ), -1 );

	return 0;
}

template<class HEADER>
int msgUSYNCH<HEADER>::
set( void*, size_t )
{
	ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUSYNCH<HEADER>::set) setting error\n" ), -1 );
}

template<class HEADER>
int msgUSYNCH<HEADER>::
execute()
{
	ACE_DEBUG(( LM_DEBUG, "(%t) executing USYNCH msg 0x%x, user: 0x%x\n", this, user_ ));
	DSMessage* msg;
	if( SDSProtocol::instance()->get( DirectoryProtocol::REGISTER, msg ) == -1 )
		ACE_ERROR_RETURN(( LM_ERROR, "[!](msgUSYNCH<HEADER>::execute) protocol error\n" ), -1 );

	msg->user( user_ );
	DSActiveUserIterator it( *SDSActiveUserRep::instance() );
	DSActiveUserIterator end( *SDSActiveUserRep::instance(), 1 );
	while( it != end )
	{
		it.acquire_read();
		ActiveUser* pUser = ( *it ).int_id_;
		const UserInfo* pInfo  = pUser->info();
		if( ! pInfo )
		{
			it++;
			it.release();
		}
		else
		{
			ActiveUserKey key = pUser->key();
			const char* ui[4]= { ( const char* )&key,
								 pInfo->first_name(),
								 pInfo->last_name(),
								 pInfo->email() };
			msg->set( ui, sizeof ui );
			it++;
			it.release();
			if( user_->send( msg ) == -1 )
				ACE_ERROR(( LM_ERROR, "[!](msgUSYNCH<HEADER>::execute) msg send error, user: 0x%x\n",
							user_ ));
		}
	}
	delete msg;
	return 0;
}

#endif
